# -*- coding:utf-8 -*-
# @Time     :2023/2/8 9:44
# @Author   :CHNJX
# @File     :testcase.py
# @Desc     :
import pytest


class Testcase1:

    def test_01(self):
        assert 1 == 1


class Testcase2:
    pytestmark = [pytest.mark.demo]

    def test_01(self):
        assert 1 != 1



# -*- coding:utf-8 -*-
# @Time     :2023/3/1 12:27
# @Author   :CHNJX
# @File     :test.py
# @Desc     :
import os
from urllib.parse import urlparse
import os


ls = os.listdir(r'G:\pythonProject\service-driver\api_object')


aa = {'s'}
aa.add('s')
print(aa)


# -*- coding:utf-8 -*-
# @Time     :2023/2/1 3:22 下午
# @Author   :CHNJX
# @File     :swagger_generate.py
# @Desc     :将swagger转换成api-object
import os
import re
import sys
from os.path import dirname, exists, join

sys.path.append(dirname(sys.path[0]))


from service_driver.loader_swagger import load_swagger
from service_driver.tenplate import Template


def generate(swagger_doc, api_dir=None):
    if not api_dir:
        api_dir = 'api_object'
    if '/' not in swagger_doc and '\\' not in swagger_doc:
        swagger_doc = 'swagger/' + swagger_doc
    swagger_data = load_swagger(swagger_doc)
    _generate_template_path(swagger_data['paths'])
    tag_path_dict = _generate_template_data(swagger_data)
    template = Template()
    for tag, paths in tag_path_dict.items():
        content = template.get_content('api.tpl', tag=tag, paths=paths)
        file_path = os.path.join(api_dir, tag.lower() + '.py')
        _write(content, file_path)


def _write(content, file_path):
    dir_ = dirname(file_path)
    if not exists(dir_):
        os.makedirs(dir_)
    with open(file_path, 'w', 'utf-8') as f:
        f.write(content)


def _get_method_attribute(value) -> str:
    attribute = ''
    if value.get('get'):
        attribute = 'get'
    elif value.get('post'):
        attribute = 'post'
    elif value.get('put'):
        attribute = 'put'
    elif value.get('delete'):
        attribute = 'delete'
    return attribute


def _transformation_parameters(parameters) -> list:
    return [param for param in parameters if
            param['name'] != 'raw' and param['name'] != 'root' and param['in'] != 'header']


# 转换json参数
def _transformation_json(parameters) -> list:
    json_list = []
    for param in parameters:
        if param.get('schema') and param['schema'].get('properties'):
            for key_name in param['schema']['properties'].keys():
                json_list.append(key_name)
    return json_list


# 拼接参数列表
def _transformation_params_list(params, json_params: list) -> list:
    params_name_list: list = [param['name'] for param in params]
    params_name_list.extend(json_params)
    return params_name_list


# 对url进行转换 适应restful风格
def _transform_url(path, method):
    path_name_list: list[str] = path.split('/')
    pat = re.compile(r'[@_!#$%^&*()<>?/\|}{~:]')
    res = {}
    if 'id' in path_name_list[-1].lower() or pat.search(path_name_list[-1]):
        res['url_param'] = re.sub('[\W_]+', '', path_name_list[-1])
        if method == 'get':
            res['name'] = 'detail'
        elif method == 'post':
            res['name'] = 'add'
        elif method == 'put':
            res['name'] = 'update'
        elif method == 'delete':
            res['name'] = 'delete'
        path_name_list.pop(-1)
        res['url'] = '/'.join(path_name_list)
    else:
        res['url_param'] = ''
        res['url'] = path
        res['name'] = path_name_list[-1].split('?')[0]
    return res


def _transformation_file(json_params) -> list:
    return [param for param in json_params if 'file' in param]


def _generate_template_path(swagger_paths):
    for path, value in swagger_paths.items():
        method_attribute = _get_method_attribute(value)
        value['method'] = method_attribute
        value['tag'] = value[method_attribute]['tags'][0]
        value['desc'] = value[method_attribute]['summary']
        parameters = value[method_attribute]['parameters'] if value[method_attribute].get('parameters') else ''
        value['parameters'] = _transformation_parameters(parameters)
        value['json'] = _transformation_json(parameters)
        value['files'] = _transformation_file(value['json'])
        value['params_list'] = _transformation_params_list(value['parameters'], value['json'])
        value.update(_transform_url(path, method_attribute))


def _generate_template_data(swagger_data) -> dict:
    tag_path_dict = {}
    for tag in swagger_data['tags']:
        tag_name: str = tag['name']
        tag_name = tag_name.replace('/', '-', -1)
        tag_path_list = {name: path for name, path in swagger_data['paths'].items() if
                         path['tag'].replace('/', '-', -1) == tag_name}
        tag_path_dict[tag_name.capitalize()] = tag_path_list
    return tag_path_dict



import os
from os.path import join, dirname

from service_driver.project_generator import project_generate
from service_driver.swagger_generate import generate


class TestProjectGenerator:
    def test_start_project(self):
        project_generate('new_project')
        assert os.path.isdir(os.path.join('new_project', "api_object"))
        assert os.path.isdir(os.path.join('new_project', "testcase"))

    def test_swagger_generate(self):
        generate(join(join(dirname(__file__), '..'), 'test/swagger/swagger.yaml'),
                 join(join(dirname(__file__), '..'), 'test/api_object'))
        assert os.path.exists(os.path.join(os.path.dirname(__file__) + "/api_object", "users2.py"))



# -*- coding:utf-8 -*-
# @Time     :2023/2/2 10:41 下午
# @Author   :CHNJX
# @File     :project_generator.py
# @Desc     :项目创造器

import sys
from os import listdir, makedirs
from os.path import dirname, exists, join

sys.path.append(dirname(sys.path[0]))

from service_driver.tenplate import Template
import click as click

group = click.Group()


def project_generate(project_name):
    """
    创建项目
    :param project_name: 项目名称
    :return: None
    """
    if exists(project_name):
        print(f'project {project_name} is already existed')
        return 1
    create_folder(project_name)
    create_folder(join(project_name, 'testcase'))
    create_folder(join(project_name, 'swagger'))
    create_folder(join(project_name, 'api_object'))
    for dir_name in listdir(project_name):
        if dir_name == 'swagger':
            continue
        cur_dir = join(project_name + '/' + dir_name, '__init__.py')
        create_file(cur_dir)
    generate_base_need(project_name)


def create_folder(path):
    makedirs(path)
    print(f'create folder {path}')


def create_file(file_path, file_content=""):
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(file_content)
    print(f"created file: {file_path}")


def generate_base_need(project_name):
    template = Template()
    api_object_dir = join(project_name, 'api_object')
    testcase_dir = join(project_name, 'testcase')
    _write(template.get_content('base_api.tpl'), join(api_object_dir, 'base_api.py'))
    _write(template.get_content('api_demo.tpl'), join(api_object_dir, 'api_demo.py'))
    _write(template.get_content('base_testcase.tpl'), join(testcase_dir, 'test_base.py'))
    _write(template.get_content('testcase_demo.tpl'), join(testcase_dir, 'testcase_demo.py'))
    _write(swagger_json, join(f'{project_name}/swagger', 'swagger_demo.json'))


def _write(content, file_path):
    dir_ = dirname(file_path)
    if not exists(dir_):
        makedirs(dir_)
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)


swagger_json = r"""
{
  "swagger": "2.0",
  "info": {
    "version": "1.0.0",
    "title": "Sample API",
    "description": "A sample API for demonstration purposes",
    "contact": {
      "name": "Your Name",
      "email": "youremail@example.com"
    }
  },
  "host": "api.example.com",
  "basePath": "/v1",
  "schemes": [
    "http"
  ],
  "tags": [
    {
      "name": "users",
      "description": "Operations related to users"
    }
  ],
  "paths": {
    "/users": {
      "get": {
        "tags": [
          "users"
        ],
        "summary": "Get a list of users",
        "description": "Returns a list of all users in the system",
        "produces": [
          "application/json"
        ],
        "responses": {
          "200": {
            "description": "Successful operation",
            "schema": {
              "type": "array",
              "items": {
                "$ref": "#/definitions/User"
              }
            }
          },
          "default": {
            "description": "Error response",
            "schema": {
              "$ref": "#/definitions/Error"
            }
          }
        }
      }
    },
    "/users/{userId}": {
      "get": {
        "tags": [
          "users"
        ],
        "summary": "Get a user by ID",
        "description": "Returns the user with the specified ID",
        "produces": [
          "application/json"
        ],
        "parameters": [
          {
            "name": "userId",
            "in": "path",
            "description": "The ID of the user to retrieve",
            "required": true,
            "type": "integer"
          }
        ],
        "responses": {
          "200": {
            "description": "Successful operation",
            "schema": {
              "$ref": "#/definitions/User"
            }
          },
          "default": {
            "description": "Error response",
            "schema": {
              "$ref": "#/definitions/Error"
            }
          }
        }
      }
    }
  },
  "definitions": {
    "User": {
      "type": "object",
      "properties": {
        "id": {
          "type": "integer"
        },
        "name": {
          "type": "string"
        },
        "email": {
          "type": "string"
        }
      }
    },
    "Error": {
      "type": "object",
      "properties": {
        "message": {
          "type": "string"
        },
        "code": {
          "type": "integer"
        }
      }
    }
  }
}
"""

if __name__ == '__main__':
    project_generate('ss')



# -*- coding:utf-8 -*-
# @Time     :2023/2/28 18:47
# @Author   :CHNJX
# @File     :writer_content.py
# @Desc     :
import os


def write(content, file_path):
    dir_ = os.path.dirname(file_path)
    if not os.path.exists(dir_):
        os.makedirs(dir_)
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)
